@@ -1,7 +1,7 @@
 # cython.* namespace for pure mode.
 from __future__ import absolute_import
 
-__version__ = "0.26.1"
+__version__ = "0.29.14"
 
 try:
     from __builtin__ import basestring
@@ -108,11 +108,14 @@ class _Optimization(object):
 cclass = ccall = cfunc = _EmptyDecoratorAndManager()
 
 returns = wraparound = boundscheck = initializedcheck = nonecheck = \
-    overflowcheck = embedsignature = cdivision = cdivision_warnings = \
-    always_allows_keywords = profile = linetrace = infer_type = \
+    embedsignature = cdivision = cdivision_warnings = \
+    always_allows_keywords = profile = linetrace = infer_types = \
     unraisable_tracebacks = freelist = \
-        lambda arg: _EmptyDecoratorAndManager()
+        lambda _: _EmptyDecoratorAndManager()
 
+exceptval = lambda _=None, check=True: _EmptyDecoratorAndManager()
+
+overflowcheck = lambda _: _EmptyDecoratorAndManager()
 optimization = _Optimization()
 
 overflowcheck.fold = optimization.use_switch = \
@@ -183,8 +186,15 @@ def declare(type=None, value=_Unspecified, **kwds):
         return value
 
 class _nogil(object):
-    """Support for 'with nogil' statement
+    """Support for 'with nogil' statement and @nogil decorator.
     """
+    def __call__(self, x):
+        if callable(x):
+            # Used as function decorator => return the function unchanged.
+            return x
+        # Used as conditional context manager or to create an "@nogil(True/False)" decorator => keep going.
+        return self
+
     def __enter__(self):
         pass
     def __exit__(self, exc_class, exc, tb):
@@ -194,6 +204,7 @@ nogil = _nogil()
 gil = _nogil()
 del _nogil
 
+
 # Emulated types
 
 class CythonMetaType(type):
@@ -383,7 +394,7 @@ py_complex = typedef(complex, "double complex")
 int_types = ['char', 'short', 'Py_UNICODE', 'int', 'Py_UCS4', 'long', 'longlong', 'Py_ssize_t', 'size_t']
 float_types = ['longdouble', 'double', 'float']
 complex_types = ['longdoublecomplex', 'doublecomplex', 'floatcomplex', 'complex']
-other_types = ['bint', 'void']
+other_types = ['bint', 'void', 'Py_tss_t']
 
 to_repr = {
     'longlong': 'long long',
@@ -418,14 +429,17 @@ for name in complex_types:
     gs[name] = typedef(py_complex, to_repr(name, name))
 
 bint = typedef(bool, "bint")
-void = typedef(int, "void")
+void = typedef(None, "void")
+Py_tss_t = typedef(None, "Py_tss_t")
 
 for t in int_types + float_types + complex_types + other_types:
     for i in range(1, 4):
-        gs["%s_%s" % ('p'*i, t)] = globals()[t]._pointer(i)
+        gs["%s_%s" % ('p'*i, t)] = gs[t]._pointer(i)
 
-void = typedef(None, "void")
-NULL = p_void(0)
+NULL = gs['p_void'](0)
+
+# looks like 'gs' has some users out there by now...
+#del gs
 
 integral = floating = numeric = _FusedType()
 
@@ -441,7 +455,7 @@ class CythonDotParallel(object):
     def parallel(self, num_threads=None):
         return nogil
 
-    def prange(self, start=0, stop=None, step=1, schedule=None, nogil=False):
+    def prange(self, start=0, stop=None, step=1, nogil=False, schedule=None, chunksize=None, num_threads=None):
         if stop is None:
             stop = start
             start = 0
